package cn.com.qmth.stmms.ms.collect.api;

import cn.com.qmth.stmms.ms.accesscontrol.config.LoginConfig;
import cn.com.qmth.stmms.ms.admin.service.DataUploadService;
import cn.com.qmth.stmms.ms.collect.dto.CollectStuDTO;
import cn.com.qmth.stmms.ms.collect.dto.CollectSubjectDTO;
import cn.com.qmth.stmms.ms.collect.dto.LoginDTO;
import cn.com.qmth.stmms.ms.commons.config.ImageCompressionConfig;
import cn.com.qmth.stmms.ms.commons.config.SystemConfig;
import cn.com.qmth.stmms.ms.commons.constant.SystemConstant;
import cn.com.qmth.stmms.ms.commons.utils.MD5Util;
import cn.com.qmth.stmms.ms.commons.utils.PictureUtil;
import cn.com.qmth.stmms.ms.commons.utils.image.ImageCompression;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import net.sf.json.JSONObject;
import org.apache.commons.codec.digest.DigestUtils;
import org.assertj.core.util.Strings;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.annotation.*;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Consumer;

/**
 * Created by dizhi on 2016/12/24.
 */
@RestController
@RequestMapping("api")
public class CollectApi {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(CollectApi.class);

    @Autowired
    private LoginConfig loginConfig;

    @Autowired
    private WorkRepo workRepo;

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private DataUploadService dataUploadService;

    @Autowired
    private SystemConfig systemConfig;

    @Autowired
    private ImageCompressionConfig compressionConfig;

    @Autowired
    private PaperRepo paperRepo;

    @RequestMapping("user/login")
    public LoginDTO login(@RequestParam String loginname, @RequestParam String password) {
        LoginDTO loginDTO = null;
        if (loginname.equals(loginConfig.adminLoginConfig().getLoginName()) && password.equals(loginConfig.adminLoginConfig().getPassword()) ||
                loginname.equals(loginConfig.clientLoginConfig().getLoginName()) && password.equals(loginConfig.clientLoginConfig().getPassword())) {
            Work activeWork = workRepo.findByActiveTrue();
            loginDTO = new LoginDTO();
            loginDTO.setExamId(activeWork.getId());
            loginDTO.setExamName(activeWork.getName());
            List<CollectSubjectDTO> collectSubjectDTOs = new ArrayList<>();
            activeWork.getSubjects().forEach(s -> {
                int id = s.getSubject().ordinal() + 1;
                CollectSubjectDTO collectSubjectDTO = new CollectSubjectDTO(id, s.getName());
                if (!Strings.isNullOrEmpty(s.getCollectConfig())) {
                    JSONObject json = JSONObject.fromObject(s.getCollectConfig());
                    collectSubjectDTO.setCollectConfig(json);
                }

                collectSubjectDTOs.add(collectSubjectDTO);
            });
            loginDTO.setSubjects(collectSubjectDTOs);
        }
        return loginDTO;
    }

    /**
     * 获取客户端账户名和密码
     *
     * @return
     */
    @RequestMapping(value = "user/getClientUser", method = RequestMethod.GET)
    public Map getClientUser() {
        Map map = new HashMap();
        map.put("clientUserName", loginConfig.clientLoginConfig().getLoginName());
        map.put("clientUserPwd", loginConfig.clientLoginConfig().getPassword());
        return map;
    }

    @RequestMapping(value = "exam/students", method = RequestMethod.GET)
    public List<CollectStuDTO> getStudents() {
        List<CollectStuDTO> list = new ArrayList<>();
        Work activeWork = workRepo.findByActiveTrue();
        Consumer<Student> consumer = (s) -> {
            CollectStuDTO collectStuDTO = new CollectStuDTO();
            collectStuDTO.setExamId(activeWork.getId());
            collectStuDTO.setExamNumber(s.getExamNumber());
            collectStuDTO.setName(s.getName());
            collectStuDTO.setSiteCode(s.getAreaName());
            collectStuDTO.setRoomCode(s.getExamNumber());
            list.add(collectStuDTO);
        };
        studentRepo.findByWorkId(activeWork.getId()).forEach(consumer);
        return list;
    }

    @RequestMapping(value = "upload/student/{subjectId}", method = RequestMethod.POST)
    public List<CollectStuDTO> saveStudent(HttpServletRequest request, @PathVariable Integer subjectId,
                                           @RequestBody CollectStuDTO[] uploadStudentArray) throws Exception {
        Subject subject = Subject.values()[subjectId - 1];
        Work activeWork = workRepo.findByActiveTrue();
        List<CollectStuDTO> list = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
        for (CollectStuDTO dto : uploadStudentArray) {
            Student student = studentRepo.findByWorkIdAndExamNumber(activeWork.getId(), dto.getExamNumber());
            if (!dto.isAbsent()) {
                dataUploadService.savePaper(student, subject, dto.isManual());
            }
            dto.setUploadTime(sdf.format(new Date()));
            list.add(dto);
        }
        return list;
    }

    File srcFile = new File("/Users/king/stmms-ms/static/thumbs/6/SX/1/1901040150.jpg"); //初始文件

    /**
     * 上传图片到服务器(图片加密)
     *
     * @param workId
     * @param subjectId
     * @param fileName
     * @param request
     * @param response
     * @throws IOException
     */
    @RequestMapping(value = "file/image/upload/{workId}/{subjectId}/{fileName}", method = {RequestMethod.GET})
    public void imageUpload(@PathVariable Long workId, @PathVariable Integer subjectId,
                            @PathVariable String fileName,
                            HttpServletRequest request, HttpServletResponse response) throws IOException {
        InputStream inputStream = null;
        OutputStream outputStream = null;
        try {
//            inputStream = request.getInputStream();
//            String md5 = request.getHeader("md5");
            inputStream = PictureUtil.getInput(srcFile);
            Student student = studentRepo.findByWorkIdAndExamNumber(workId, fileName);
            Subject subject = Subject.values()[subjectId - 1];
            //保存遮盖图+文件名加密
            String savePath = systemConfig.getImageDir() + File.separator + workId + File.separator + subject
                    + File.separator + student.getAreaCode();
            File out = new File(savePath);
            if (!out.exists()) {
                out.mkdirs();
            }

            String imageMd5 = getImageRuleMd5(workId, subject.ordinal(), student.getAreaCode(), student.getExamNumber(), student.getId(), SystemConstant.IMAGE);
//            暂不用Aes加密，因为没用到反解
//            StringBuffer stringBuffer = new StringBuffer(String.valueOf(workId)).append(subject.ordinal()).append(student.getAreaCode()).append(student.getExamNumber()).append(student.getId()).append(image);
//            String rule = stringBuffer.toString();
//            LOGGER.info("rule:{},length:{}", rule, rule.length());
//            rule = String.format("%016d", Long.parseLong(rule));
//            LOGGER.info("rule补零后:{},length:{}", rule, rule.length());
//            String aesStr = AesUtil.encoder(student.getExamNumber(), stringBuffer.toString(), rule);
//            LOGGER.info("aesStr:{}", aesStr);
//            File outFile = new File(savePath + File.separator + aesStr + ".jpg");
            File outFile = new File(savePath + File.separator + imageMd5 + ".jpg");
            LOGGER.info("image存放目录:{}", outFile.getPath());
            outputStream = new FileOutputStream(outFile);
            int dataOfFile = 0;
            while ((dataOfFile = inputStream.read()) > -1) {
                outputStream.write(dataOfFile ^ SystemConstant.SECRET_KEY);
            }

            // 生成缩略图
            String thumbDir = systemConfig.getThumbDir() + File.separator + workId + File.separator + subject
                    + File.separator + student.getAreaCode();
            File thumb = new File(thumbDir);
            if (!thumb.exists()) {
                thumb.mkdirs();
            }
            BufferedImage bufferedImage = ImageCompression.compress(outFile, compressionConfig);
//            String thumbMd5 = getImageRuleMd5(workId, subject.ordinal(), student.getAreaCode(), student.getExamNumber(), student.getId(), SystemConstant.THUMB);
            File thumbFile = new File(savePath + File.separator + imageMd5 + ".jpg");
            ImageIO.write(bufferedImage, "jpg", thumbFile);
//            FileInputStream in = new FileInputStream(outFile);
//            String sliceMD5 = DigestUtils.md5Hex(in);
//            if (!md5.equalsIgnoreCase(sliceMD5)) {
//                throw new Exception("图片md5值不一致");
//            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(inputStream)) {
                inputStream.close();
            }
            if (Objects.nonNull(inputStream)) {
                outputStream.flush();
                outputStream.close();
            }
        }
    }

    /**
     * 从服务器上下载图片(图片解密)
     *
     * @param workId
     * @param subjectId
     * @param studentId
     * @param imageType
     * @param request
     * @param response
     * @throws IOException
     */
    @RequestMapping(value = "file/image/download/{workId}/{subjectId}/{studentId}/{imageType}", method = {RequestMethod.GET})
    public void imageDownload(@PathVariable Long workId, @PathVariable Integer subjectId,
                              @PathVariable Long studentId, @PathVariable Integer imageType,
                              HttpServletRequest request, HttpServletResponse response) throws IOException {
        //读取路径下面的文件
        InputStream in = null;
        OutputStream outputStream = null;
        try {
            Student student = studentRepo.findOne(studentId);
            Subject subject = Subject.values()[subjectId - 1];
            String savePath = systemConfig.getImageDir() + File.separator + workId + File.separator + subject
                    + File.separator + student.getAreaCode();
//            暂不用Aes解密，因为没用到反解
//            StringBuffer stringBuffer = new StringBuffer(String.valueOf(workId)).append(subject.ordinal()).append(student.getAreaCode()).append(student.getExamNumber()).append(student.getId()).append(image);
//            String savePath = systemConfig.getImageDir() + File.separator + workId + File.separator + subject
//                    + File.separator + student.getAreaCode();
//            String rule = stringBuffer.toString();
//            LOGGER.info("rule:{},length:{}", rule, rule.length());
//            rule = String.format("%016d", Long.parseLong(rule));
//            LOGGER.info("rule补零后:{},length:{}", rule, rule.length());
//            String aesStr = AesUtil.encoder(student.getExamNumber(), stringBuffer.toString(), rule);
//            String examNumber = AesUtil.decoder(aesStr, stringBuffer.toString(), rule);
//            LOGGER.info("解密后的文件名称:{}", examNumber);
            String md5 = getImageRuleMd5(workId, subject.ordinal(), student.getAreaCode(), student.getExamNumber(), student.getId(), imageType);
//            String ext = file.getName().substring(file.getName().indexOf(".") + 1);
            //判断图片格式,设置相应的输出文件格式
//            if (ext.equalsIgnoreCase("jpg")) {
            response.setContentType("image/jpg");
//            } else if (ext.equalsIgnoreCase("png")) {
//                response.setContentType("image/png");
//            }
            File file = new File(savePath + File.separator + md5 + ".jpg");
            //读取指定路径下面的文件
            in = new FileInputStream(file);
            outputStream = new BufferedOutputStream(response.getOutputStream());
            int dataOfFile = 0;
            while ((dataOfFile = in.read()) > -1) {
                outputStream.write(dataOfFile ^ SystemConstant.SECRET_KEY);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            //强制将缓存区的数据进行输出
            if (Objects.nonNull(outputStream)) {
                outputStream.flush();
                //关流
                outputStream.close();
            }
            if (Objects.nonNull(in)) {
                in.close();
            }
        }
    }

    /**
     * 上传裁切原图并压缩
     *
     * @param workId
     * @param subjectId
     * @param fileName
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "file/ms-slice/{workId}/{subjectId}/{fileName}", method = {RequestMethod.PUT, RequestMethod.POST})
    public void upload(@PathVariable Long workId, @PathVariable Integer subjectId,
                       @PathVariable String fileName,
                       HttpServletRequest request, HttpServletResponse response) throws Exception {
        InputStream inputStream = request.getInputStream();
        Student student = studentRepo.findByWorkIdAndExamNumber(workId, fileName);
        Subject subject = Subject.values()[subjectId - 1];
        //保存遮盖图
        String savePath = systemConfig.getImageDir() + File.separator + workId + File.separator + subject
                + File.separator + student.getAreaCode();
        File outFile = saveImage(student, inputStream, savePath);

        // 生成缩略图
        String thumbDir = systemConfig.getThumbDir() + File.separator + workId + File.separator + subject
                + File.separator + student.getAreaCode();
        File thumb = new File(thumbDir);
        if (!thumb.exists()) {
            thumb.mkdirs();
        }
        BufferedImage bufferedImage = ImageCompression.compress(outFile, compressionConfig);
        String thumbFileName = thumbDir + File.separator + student.getExamNumber() + ".jpg";
        ImageIO.write(bufferedImage, "jpg", new File(thumbFileName));

        String md5 = request.getHeader("md5");
        FileInputStream in = new FileInputStream(outFile);
        String sliceMD5 = DigestUtils.md5Hex(in);
        if (!md5.equalsIgnoreCase(sliceMD5)) {
            throw new Exception("图片md5值不一致");
        }
        in.close();
//        ImageCompression.imageThumbnail(outFile,new File(thumbFileName),compressionConfig);
    }

    private File saveImage(Student student, InputStream inputStream,
                           String savePath) throws IOException, FileNotFoundException {
        File out = new File(savePath);
        if (!out.exists()) {
            out.mkdirs();
        }
        File outFile = new File(savePath + File.separator + student.getExamNumber() + ".jpg");
        FileCopyUtils.copy(inputStream, new FileOutputStream(outFile));
        return outFile;
    }

    /**
     * 上传裁切原图并压缩
     *
     * @param workId
     * @param subjectId
     * @param fileName
     * @param request
     * @param response
     * @throws IOException
     */
    @RequestMapping(value = "file/ms-sheet/{workId}/{subjectId}/{fileName}", method = {RequestMethod.PUT, RequestMethod.POST})
    public void uploadsheet(@PathVariable Long workId, @PathVariable Integer subjectId,
                            @PathVariable String fileName,
                            HttpServletRequest request, HttpServletResponse response) throws Exception {
        Student student = studentRepo.findByWorkIdAndExamNumber(workId, fileName);
        Subject subject = Subject.values()[subjectId - 1];
        InputStream inputStream = request.getInputStream();
        //保存原图图
        String savePath = systemConfig.getSheetDir() + File.separator + workId + File.separator + subject
                + File.separator + student.getAreaCode();
        File outFile = saveImage(student, inputStream, savePath);

        String md5 = request.getHeader("md5");
        FileInputStream in = new FileInputStream(outFile);
        String sheetMD5 = DigestUtils.md5Hex(in);
        if (!md5.equalsIgnoreCase(sheetMD5)) {
            throw new Exception("图片md5值不一致");
        }
        in.close();
    }

    @RequestMapping(value = "subject/collect-config", method = RequestMethod.POST)
    public boolean updateConfig(@RequestBody CollectSubjectDTO collectSubjectDTO) {
        Subject subject = Subject.values()[collectSubjectDTO.getSubjectId() - 1];
        Work activeWork = workRepo.findByActiveTrue();
        MarkSubject markSubject = markSubjectRepo.findOne(activeWork.getId() + "-" + subject.toString());
        String config = Optional.ofNullable(collectSubjectDTO.getCollectConfig())
                .map(Object::toString)
                .map(s -> s.substring(1, s.length() - 1))
                .map(s -> s.replaceAll("\\\\", "")).get();
        markSubject.setCollectConfig(config);
        markSubjectRepo.save(markSubject);
        return true;
    }

    /**
     * 生成图片md5规则
     *
     * @param workId
     * @param subjectId
     * @param areaCode
     * @param examNumber
     * @param studentId
     * @param imageType
     * @return
     * @throws Exception
     */
    public String getImageRuleMd5(Long workId, int subjectId, String areaCode, String examNumber, Long studentId, int imageType) throws Exception {
        //暂不用Aes解密，因为没用到反解
        StringBuffer stringBuffer = new StringBuffer(String.valueOf(workId)).append(subjectId).append(areaCode).append(examNumber).append(studentId).append(imageType);
        String rule = stringBuffer.toString();
        LOGGER.info("rule:{},length:{}", rule, rule.length());
        rule = String.format("%025d", Long.parseLong(rule));
        LOGGER.info("rule补零后:{},length:{}", rule, rule.length());
        return MD5Util.encoder(rule);
    }
}
